
CreateDate<-function(dat, format="%d/%m/%y") {

  if (format=="%d/%m/%y") {
    if (nchar(dat[1])==8) {
  	 strptime(dat, "%d/%m/%y")
	  } else {
  	 strptime(dat, "%d/%m/%Y")
	  }
	} else {
	  strptime(dat, format)
	}
}

GetCurrentDate<-function()	{
	strptime(format(Sys.time(), "%d/%m/%y"), "%d/%m/%y")
}
	
DateDiff<- function(dat1, dat2) {

	
	if (is.date(dat1)==FALSE) dat1<-CreateDate(dat1)
  if (is.date(dat2)==FALSE) dat2<-CreateDate(dat2)
  as.integer(round(difftime(dat2, dat1, units="days")))

}


is.date<-function(dat) {

  out<-FALSE
  if (class(dat[1])[1]=="POSIXt" | class(dat[1])[1]=="POSIXct" | class(dat[1])[1]=="POSIXlt") out<-TRUE
  out
}


EqGen <-
function (
        spotPrice,
        dividendReturn=0,                        
        corr=1,                                    
        nSim=10,			
        date=NA,	               	
                                        
        years=NA,                      
        dt=NA,                          
        times=NA,
	      shift=0,                        
	      curv,                           
                                        
	      a=NA,
	      sigma=0.1,
	      delta=0,                        
        seed=sample(32000, size=1)
) {

  set.seed(seed)
  nEquities=length(spotPrice)
  if (!is.matrix(corr)) { corr<-diag(1-corr, nEquities) + corr }

  fspotRate<-function(dT) {
    fs<-rep(0, nEquities)
    for (i in (1:nEquities))  {
      fs[i]<-splinefun(matrix(c(curv[,1], curv[, i+1]), ncol=2, byrow=FALSE))(dT)
    }
    fs
  }

  fforwRate<-function(dT0, dT1) {
    (fspotRate(dT1) * dT1 - fspotRate(dT0) * dT0)/(dT1-dT0)
  }

  if (is.na(times)[1]) { times<-seq(shift+dt,shift+years,dt) }
  equity_prices<-array(0, dim=c(nEquities,nSim, length(times)+1))     

  rates<-matrix(0, nEquities, length(times))
  rates[,1]<-fspotRate(times[1])

  if (length(times)!=1) {
    for (i in (2:length(times)))  {
      rates[,i]<-fforwRate(times[i-1],times[i])
    }
  }


  equity_prices[,,1]<-spotPrice
  times<-c(0, times)
  for (t in 2:(length(times))) {
    dT<-(times[t]-times[t-1])
    equity_prices[,,t]<-equity_prices[,,t-1] * exp((rates[,t-1] - dividendReturn - 0.5 * sigma * sigma) * dT
                                                    + sigma * t(mvrnorm(nSim, rep(0, nEquities), corr)) * sqrt(dT))
  }

  if (nEquities==1) {
    equity_prices[1,,]
  }  else {
    equity_prices
  }

}


CorrelationMatrix<-function(Lines) {
  if (is.numeric(Lines[1,1])==FALSE) {	Lines<-Lines[-1,]	}
	Lines<-log(Lines[-1,]/ Lines[-length(Lines[,1]),])
  cor(Lines)
}

VolatilityMatrix<-function(Lines, Lambda=0.97) {

  if (is.numeric(Lines[1,1])==FALSE) {	Lines<-Lines[-1,]	}
	Lines<-log(Lines[-1,]/ Lines[-length(Lines[,1]),])
	J<-as.data.frame(matrix(rep(0, dim(Lines)[2]*3), nrow=3))

  J[1,]<-sqrt(apply(Lines, 2, var))
	J[2,]<-sqrt(apply(Lines, 2, var))*sqrt(252)      
	J[3,]<-EVol(Lines,Lambda)
	as.matrix(J)
}


EVol <- function(mRada, Lambda=0.97) {  

  if (is.matrix(mRada)==FALSE) {
    mRada<-as.matrix(mRada, ncol=1)
  }                 	
 
    
  mRadaAve = apply(mRada,2,mean)  
  mRada = t(t(mRada) - mRadaAve)

  Lambda.vector<-rep(0, dim(mRada)[1])
  Lambda.power.vector<-rep(0, dim(mRada)[1])
  for ( i in 1: (dim(mRada)[1]))
  {
    Lambda.power.vector[i]<-Lambda^i
  }
  Lambda.vector<-Lambda.power.vector/sum(Lambda.power.vector)
 	Volatility<-sqrt(apply((matrix(rep(Lambda.vector, dim(mRada)[2]), ncol=dim(mRada)[2], byrow=FALSE)*mRada^2), 2, sum))
  Volatility * sqrt(252)     
}



CreateCalendar <- function(startDate, len=1, byy=1, unita="month", form="string", convention="")  {
	if (is.date(startDate)==FALSE)  {startDate <- CreateDate(startDate)}
	sD <- startDate + 2*60*60

  res<-CreateDate(format.Date(seq(sD, len=len, by=paste(byy, unita, sep=" ")), "%d/%m/%Y"))

  if  (convention=="ModFol") {
      op<-(substr(weekdays(res), 1, 2) == "so"|substr(weekdays(res), 1, 2) == "Sa") * 2
      op<-(substr(weekdays(res), 1, 2) == "ne"|substr(weekdays(res), 1, 2) == "Su") * 1 + op
  }  else  if  (convention=="ModPrec") {
      op<-(substr(weekdays(res), 1, 2) == "so"|substr(weekdays(res), 1, 2) == "Sa") * -1
      op<-(substr(weekdays(res), 1, 2) == "ne"|substr(weekdays(res), 1, 2) == "Su") * -2 + op
  }
  res<-res + op * 60 * 60 * 24 + 60*60 
	if (form=="string") {
    DestroyDate(res)
	}  else  {
    res
	}
}

DestroyDate<-function(dat, mask="%d/%m/%Y")  {
  if (is.date(dat)) {
    res<-strftime(as.POSIXlt(dat), mask)
  } else {
    res<-"NA"
  }
  res
}